@@ -1,4 +1,4 @@
 utils::globalVariables(
   c("processed", "Gene", "Counts", "metric", "score", "original",
-    "scaled_score", "gene", "cluster", "mean_expression", "avg_cluster_exp")
+    "scaled_score", "gene", "cluster", "mean_expression", "Avg_exp")
 )
